<?php
/*
 +-------------------------------------------------------------------------+
 |                                                                         |
 | This program is free software; you can redistribute it and/or           |
 | modify it under the terms of the GNU General Public License             |
 | as published by the Free Software Foundation; either version 2          |
 | of the License, or (at your option) any later version.                  |
 |                                                                         |
 | This program is distributed in the hope that it will be useful,         |
 | but WITHOUT ANY WARRANTY; without even the implied warranty of          |
 | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           |
 | GNU General Public License for more details.                            |
 +-------------------------------------------------------------------------+
*/
/*******************************************************************************
    Author ......... Jean-Michel Pepin classes plugin for netwrok devices (PEPJ in forum.cacti.net & http://cactiusers.org/forums/)
    Modified ....... .
    Program ........ class definition for network devices in PHP for Cacti or PHP software
    version ........ 0.36
    
    congratulation: For the TFTP part theses classes are based on the work of Pancho http://www.pancho.org
    & cacti http://www.cacti.net where I've picked ideas or adapt some functions, 
    Thanks to all the developpers of cacti and pancho who make my work much easier for TFTP.

reference http://www.nudesignteam.com/mib.html
reference http://www.oidview.com/mibs
http://support.ipmonitor.com/mibs_byoidtree.aspx
*******************************************************************************/

/* ----------------------- BEGIN for independancy reasons -------------------------------
   ********** TO READ ************ */
/* the following variables & functions should be set or created or included
*/
/*
$sharednetworkclasspath = "path_to_the_sharednetworkclass";

// I you need debug or log define you log function like this one below -- if you don't define it this one will be created
function networkclass_debug($message) {
  global $networkclass_isdebug,$networkclass_islog,$networkclass_debugfile;
  if ($networkclass_isdebug) print($message);
  if ($networkclass_islog) {
    $handletest = fopen($networkclass_debugfile, "a");
    if ($handletest) fwrite($handletest,$message);
    if ($handletest) fclose($handletest);
  } 
}
// example for CACTI you can create the log function like this
function networkclass_debug($message) {
  global $networkclass_isdebug,$networkclass_islog,$networkclass_debugfile;
  if ($networkclass_isdebug) print($message);
  if ($networkclass_islog) debug_log_insert(....);
}

*/
/* ----------------------- END for independancy reasons ------------------------------- */
global $networkclass_max_class_load,$networkclass_tab_class_loaded;
$networkclass_max_class_load=4;
$networkclass_tab_class_loaded=array();

global $networkclass_isdebug,$networkclass_islog,$networkclass_debugfile;
$networkclass_isdebug=false;
$networkclass_islog=false;
$networkclass_debugfile=(isset($sharednetworkclasspath))?($sharednetworkclasspath."/"."DEBUG.txt"):"DEBUG.txt";
if (!function_exists("networkclass_debug") ) {
 function networkclass_debug($message) {
  global $networkclass_isdebug,$networkclass_islog,$networkclass_debugfile;
  if ($networkclass_isdebug) print($message);
  if ($networkclass_islog) {
    $handletest = fopen($networkclass_debugfile, "a");
    if ($handletest) fwrite($handletest,$message);
    if ($handletest) fclose($handletest);
  }
 }
}

include("snmpclass.php");

include("snmp_default.class.php");
include("snmp_cisco.class.php");
include("snmp_foundry.class.php");
include("snmp_cabletron.class.php");
include("snmp_hp.class.php");
include("snmp_nortel.class.php");
include("snmp_avaya.class.php");
include("snmp_a3com.class.php");
include("snmp_dell.class.php");
include_once("xml.php");

class networkclass
{
   /*
   * Class loading function.
   */
       var $module;
       var $param, $ini=false, $active= false, $errormsg="";
       var $description="",$vendor="",$oiddevice="";
       var $snmp,$ishost=true,$inittftp=false;
       
       function LoadClass($loadModule)
       {
       	  if (!$this->active) $this->check(true);
       	  $snmpok = $this->active;
       	  if ($snmpok) {
       	    $this->setvendor();
       	    $fullloadModule =($this->vendor!=="") ? ($this->vendor.$loadModule): ("default".$loadModule);
            $this->module = new $fullloadModule; // this is much better
       	    $this->DeviceIsHost($this,true);
          }
          return $snmpok;
       }
       function Init($objsess,$path_snmpbinary="",$snmpget="",$snmpgetnext="",$snmpwalk="",$snmpset="",$snmpbulkwalk="",$snmpbin_version="")
       {
	  $this->snmp=new snmpclass;
          if ($this->snmp->Init($objsess,$path_snmpbinary,$snmpget,$snmpgetnext,$snmpwalk,$snmpset,$snmpbulkwalk,$snmpbin_version)) {
          	 $ret=true;
          	 $this->param=&$this->snmp->param;
          } else $ret=false;
          $error = $this->errormsg;
	  $this->active = false;
	  $this->errormsg = "";
	  $this->result=$ret;
	  $this->ini = $ret;
       }
       /* return if the device is a host */
       function DeviceIsHost($init=false) {
         $result=$this->module->DeviceIsHost($this,$init);
         return $result;
       }
       function InitTftp($objtftp)
       {
       	  if (($this->ini)&&($this->active) && ($this->vendor!="")) {
            // the initialisation depends on the class";
            // but we try to find a solution for all
       	    if ($objtftp[0]!="start") $objtftp[0] = "run";
       	    $this->tftpparam= array("startup" => $objtftp[0]
       	 		,"source"=> $objtftp[1]    // from tftp, start, run
       	 		,"tftpip"=> $objtftp[2]
       	 		,"filesrc"=> $objtftp[3]
       	 		,"filedest"=> $objtftp[4]
       	 		,"binary"=> isset($objtftp[5])? $objtftp[5]:0);
       	    if (($objtftp[1]!="") && ($objtftp[2]!="")) $this->inittftp = true;
       	    else $this->inittftp = false;
	    $this->retcode = $this->inittftp;
	    $this->errormsg = "";
	    $this->result= "";
       	  } else {
		$this->retcode = false;
		$this->result= "";
		$this->errormsg = "Error: object could not be initialized!";
	  }
       }
       function check($initial=false)
       {
       	  $this->active = false;
       	  $this->error= "";
          $ret = false;
       	  if ($this->ini) {
            networkclass_searchinMIB("sysObjectID",$valmib);
            $this->active = networkclass_give_snmp($this,$valmib);
            $this->errormsg= ($this->active)?"":$this->errormsg;
            if ($this->active) {
            	$this->oiddevice = $this->result;
            }
            $ret = $this->retcode;
          }
          return $ret;
       }
       function description($renew=false)
       {
       	  $this->errormsg= "";
       	  if (($this->ini)&&($this->active)) {
       	    if ($renew || ($this->description=="")) {
       	    	$ret = $this->give("sysDescr");
       	    	$s_sysDescr= $this->result;
		$this->description= $s_sysDescr;
            	if (!$this->retcode) $this->errormsg= $s_sysDescr;
		$this->result= $s_sysDescr;
            } else {
	    	// we use the result of last request
		$this->result= $this->description;
		$this->retcode = ($this->description!=="");
            }
          }
       }
       function setvendor($renew=false)
       {
       	  global $toidvendor,$toidtovendor;
       	  $this->errormsg= "";
       	  if (($this->ini)&&($this->active)) {
       	    if ($renew || ($this->vendor=="")) {
		$this->retcode = networkclass_discovervendor($this,$vendoroid);
       	    	$this->oiddevice = $vendoroid;
            	// $vendortemp= array_search($vendoroid,$toidvendor);
            	$vendortemp= (array_key_exists($vendoroid,$toidtovendor)) ? $toidtovendor[$vendoroid]: false;
            	$vendor= ($vendortemp==false) ? "default":$vendortemp;
		$this->vendor= $vendor;
            	if (!$this->retcode) $this->errormsg= $vendor;
		$this->result= $vendor;
	    } else {
	    	// we use the result of last request
		$this->result= $this->vendor;
		$this->retcode = ($this->vendor!=="");
	    }
          }
       }
       function get($mib)
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->get($mib);
	  $this->result  = $this->snmp->result;
	  $this->retcode = $this->snmp->retcode;
	  $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function getnext($mib)
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->getnext($mib);
	  $this->result  = $this->snmp->result;
	  $this->retcode = $this->snmp->retcode;
	  $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function walk($mib,$type="s")
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->walk($mib,$type);
	  $this->result  = $this->snmp->result;
	  $this->retcode = $this->snmp->retcode;
	  $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function set($mib,$typ="",$val="")
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->set($mib,$typ,$val);
	  $this->result  = $this->snmp->result;
	  $this->retcode = $this->snmp->retcode;
	  $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function set_list($tab_oid_value)
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->set_list($tab_oid_value);
	  $this->result  = $this->snmp->result;
	  $this->retcode = $this->snmp->retcode;
	  $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function snmphostname($hostname="")
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->ssnmphostname($hostname);
	  $this->result  = $this->snmp->result; $this->retcode = $this->snmp->retcode; $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function snmpversion($newversion)
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->snmpversion($newversion);
	  $this->result  = $this->snmp->result; $this->retcode = $this->snmp->retcode; $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function setsnmpcommunity($community="")
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->setsnmpcommunity($community);
	  $this->result  = $this->snmp->result; $this->retcode = $this->snmp->retcode; $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function snmptimeout($timeout="",$ini=false)
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->snmptimeout($timeout,$ini);
	  $this->result  = $this->snmp->result; $this->retcode = $this->snmp->retcode; $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function snmpport($port="",$ini=false)
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->snmpport($port,$ini);
	  $this->result  = $this->snmp->result; $this->retcode = $this->snmp->retcode; $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function snmpuserpassword($user="",$password="",$initial=false)
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->snmpuserpassword($user,$password,$initial);
	  $this->result  = $this->snmp->result; $this->retcode = $this->snmp->retcode; $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function snmpretries($retries="",$ini=false)
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->snmpretries($retries,$ini);
	  $this->result  = $this->snmp->result; $this->retcode = $this->snmp->retcode; $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
       function snmpmethod($method="",$path_binary="",$get="",$getnext="",$walk="",$set="",$binversion="")
        //this function exist only to make easier for the user of this class
       {
       	  $ret = $this->snmp->snmpmethod($method,$path_binary,$get,$getnext,$walk,$set,$binversion);
	  $this->result  = $this->snmp->result; $this->retcode = $this->snmp->retcode; $this->errormsg= $this->snmp->errormsg;
       	  return $ret;
       }
        function close()
       {
       	  if ($this->ini) $this->snmp->close();
          unset($this->snmp);
      	  $this->ini =false;
       }
       function RunFunction($FunctionName)
       {
       	  if (($this->ini)&&($this->active)) {
             $this->module->$FunctionName($this);
          } else {
            $this->errormsg="Error device ".(($this->ini)?"not initialised.":"not active.");
            $this->result ="";
            $this->retcode=0;
         }
       }
       function priv($FunctionName,$param="")
       {
          $ret = 0;
          $this->result = array();
          $this->retcode = 0;
       	  if (($this->ini)&&($this->active)) {
       	     if ($this->vendor!="") 
       	       $ret= $this->module->priv($this,$FunctionName,$param);
       	     else {
              $this->errormsg="Error vendor not set";
       	     }
          } else {
            $this->errormsg="Error device ".(($this->ini)?"not initialised.":"not active.");
          }
          return $ret;
       }
       function give($mibName,$type="s")
       {
       	  if (($this->ini)&&($this->active)) {
             networkclass_searchinMIB($mibName,$valmib);
             $ret = networkclass_give_snmp($this,$valmib,$type);
          } else {
            $this->result = "";
            $this->errormsg="Error device ".(($this->ini)?"not initialised.":"not active.");
            $this->retcode = 0;
            $ret = 0;
          }
          return $ret;
       }
       function setdebug($isdebug=false,$islog=false,$debugfile="")
       {
       	  global $networkclass_isdebug,$networkclass_islog,$networkclass_debugfile;
       	  $networkclass_isdebug=$isdebug;
       	  $networkclass_islog=$islog;
       	  if ($debugfile!="") $networkclass_debugfile=$debugfile;
       }

       function ListFunctions()
       {
   	  $functionarray=array(
   	        "classindependent"=> array("Init","showfunction","description","close"
   	            ,"get","getnext","walk","set","set_list","setdebug"
   	            ,"snmphostname","snmpversion","setsnmpcommunity","snmptimeout","snmpport"
   	            ,"snmpuserpassword","snmpretries","snmpmethod"),
   	  	"classdependant" => array("LoadClass","give"
   	  	    ,"Duplex","Trunk","Channel","LACP","PaGP","VLAN"
   	  	    ,"CPU","Temperature","DeviceIsHost","Checktypdevice"
   	  	    ,"MAC", "ARP", "STPchanges","Fan","RAM","STPchanges"
   	  	    ,"RunFunction","priv","InitTftp","tftp")
   	  	);
          return $functionarray;       
       }
}

/* ************************************************************************
 ARP : for the VlanID should return an array of IPs addresses like
 $result[interfaceindex][indexIPs] -> IP address where interfaceindex is the inteface index
*/
function networkclass_ARP($objdevice,$vlanID="") {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
 $a_xmlmib = array(array("source"=>"RFC1213-MIB.xml"));
 $snmpok=0;
 $snmpok2 =0;
 $msg = "Error: ARP function RFC1213 not enabled or not supported.";
 $result = array();
 $tIPs = array();
 $tIfindx = array();
 searchin_xmlMIB($a_xmlmib,"ifIndex",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG ifIndex function MIB ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   if (($vlanID!="") && ($vlanID!="1")) {
      $oldcommunity = $objdevice->snmp->setsnmpcommunity();
      $newcommunity = $objdevice->snmp->setsnmpcommunity($oldcommunity."@".$vlanID);
   }
   networkclass_give_snmp($objdevice,$valmib);
   if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$tIfindx);
   $snmpok1=$objdevice->retcode;
   if ($snmpok1) {
     // search the IP addr
     searchin_xmlMIB($a_xmlmib,"ipAdEntIfIndex",$valmib);
    networkclass_give_snmp($objdevice,$valmib);
    $snmpok2=$objdevice->retcode;
    if ($snmpok2) {
        remove_oid_for_find_index($valmib["oid"],$objdevice->result,$tempresult);
        foreach ($tempresult as $dx => $val) {
         if (array_key_exists($val,$tIfindx)) {
           $tIPs[$tIfindx[$val]][] = $dx;
      	 }
        }
        $msg = "";
    }
   }
   if (($vlanID!="") && ($vlanID!="1")) {
      $oldcommunity = $objdevice->snmp->setsnmpcommunity($oldcommunity);
   }
 }
 $snmpok = $snmpok2;
 foreach ($tIPs as $ifndx => $IP) {
  $result[$ifndx][] = $IP;
 }
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG ARP: IPs ipAdEntIfIndex result ->".print_r($result,true)."\n");
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 MAC : for the VlanID should return an array of MAC addresses like
 $result[interfaceindex][indexMAC] -> Macaddress where interfaceindex is the inteface index
*/
function networkclass_MAC($objdevice,$vlanID="") {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
 $a_xmlmib = array(array("source"=>"RFC1493-BRIDGE-MIB.xml"));
 $snmpok=0;
 $snmpok2 =0;
 $msg = "Error: RFC1493 not enabled or not supported.";
 $result = array();
 $tMAC = array();
 $tIfindx = array();
 searchin_xmlMIB($a_xmlmib,"dot1dBasePortIfIndex",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG dot1dBasePortIfIndex function MIB ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   if (($vlanID!="") && ($vlanID!="1")) {
      $oldcommunity = $objdevice->snmp->setsnmpcommunity();
      $newcommunity = $objdevice->snmp->setsnmpcommunity($oldcommunity."@".$vlanID);
   }
   networkclass_give_snmp($objdevice,$valmib);
   if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$tIfindx);
   $snmpok1=$objdevice->retcode;
   if ($snmpok1) {
     // search the MAC addr
     searchin_xmlMIB($a_xmlmib,"dot1dTpFdbPort",$valmib);
    networkclass_give_snmp($objdevice,$valmib);
    $snmpok2=$objdevice->retcode;
    if ($snmpok2) {
        remove_oid_for_find_index($valmib["oid"],$objdevice->result,$tempresult);
        foreach ($tempresult as $dx => $val) {
         if (array_key_exists($val,$tIfindx)) {
           $tMAC[$tIfindx[$val]][] = $dx;
      	 }
        }
        $msg = "";
    }
   }
   if (($vlanID!="") && ($vlanID!="1")) {
      $oldcommunity = $objdevice->snmp->setsnmpcommunity($oldcommunity);
   }
 }
 $snmpok = $snmpok2;
 foreach ($tMAC as $ifndx => $maclist) {
  foreach ($maclist as $ndx => $mac) {
   $temp = array("0","0","0","0","0","0"); // only in case the value is not complete
   $temp = preg_split("/\./",$mac);
   $result[$ifndx][] = sprintf("%02x-%02x-%02x-%02x-%02x-%02x"
                       ,intval($temp[0]),intval($temp[1]),intval($temp[2]),intval($temp[3]),intval($temp[4]),intval($temp[5]));
  }
 }
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG MAC dot1dTpFdbPort result ->".print_r($result,true)."\n");
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 STPchanges : for the VlanID should return an array of 
 $result["TopChanges"] -> The total number of topology changes detected by this bridge since the management entity was last reset or initialized.
 $result["TimeSinceTopologyChange"] -> The time (in hundredths of a second) since the last time a topology change was detected by the bridge entity.
*/
function networkclass_STPchanges($objdevice,$vlanID="") {
 global $networkclass_isdebug,$networkclass_islog;
//force debug $objdevice->setdebug(true,true);
 $a_xmlmib = array(array("source"=>"RFC1493-BRIDGE-MIB.xml"));
 $snmpok=0;
 $msg = "Error: RFC1493 not enabled or not supported.";
 $result = array();
 if (($vlanID!="") && ($vlanID!="1")) {
      $oldcommunity = $objdevice->snmp->setsnmpcommunity();
      $newcommunity = $objdevice->snmp->setsnmpcommunity($oldcommunity."@".$vlanID);
 }
 searchin_xmlMIB($a_xmlmib,"dot1dStpTopChanges",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG dot1dStpTopChanges MIB ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok=$objdevice->retcode;
   if ($snmpok) {
     $result["TopChanges"] = $objdevice->result;
     $msg = "";
   } else {
     $result["TopChanges"] = "";
     $msg = "Error: dot1dStpTopChanges no answer.";
   }
 }
 if ($snmpok) {
  searchin_xmlMIB($a_xmlmib,"dot1dStpTimeSinceTopologyChange",$valmib);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG dot1dStpTimeSinceTopologyChange MIB ->".print_r($valmib,true)."\n");
  if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok=$objdevice->retcode;
   if ($snmpok) {
     $result["TimeSinceTopologyChange"] = $objdevice->result;
     $msg .= "";
   } else {
     $result["TimeSinceTopologyChange"] = "";
     $msg .= "Error: dot1dStpTimeSinceTopologyChange no answer.";
   }
  }
 }
 if (($vlanID!="") && ($vlanID!="1")) {
      $oldcommunity = $objdevice->snmp->setsnmpcommunity($oldcommunity);
 }
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG STPchanges result ->".print_r($result,true)."\n");
 $objdevice->result= $result;
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

/* ************************************************************************
 LACP : should return an array like in order to be independant for vendors
 per default IEEE8023-LAG-MIB
 $result["AdminKey"][ifIndex] = AdminKey of LaCP channel;   "dot3adAggPortActorAdminKey"
 $result["OperKey"][ifIndex] = OperKey of LaCP channel;     "dot3adAggPortActorOperKey"
 $result["PortActorAdminState"][ifIndex] = Port Selected in which index of the LaCP channel
 $result["PortActorOperState"][ifIndex] = Port Attached in which index of the LaCP channel
 */
function networkclass_LACP($objdevice) {
 global $networkclass_isdebug,$networkclass_islog;
//$objdevice->setdebug(true,true); // for manually debug
 $LACPAdminKey= array(); 
 $LACPOperKey = array(); 
 $PortActorAdminState = array();
 $PortActorOperState = array();
 $LACPPortListEntry = array();
 $LACPPortListEntry2 = array();
 $snmpok=0;
 $msg = "Error: MIB not exist.";
 $a_xmlmib = array(array("source"=>"IEEE8023-LAG-MIB.xml"));
// if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG LACP Load MIB ->".print_r($a_xmlmib,true)."\n");
 /* search the LACP information */
 searchin_xmlMIB($a_xmlmib,"dot3adAggPortActorAdminKey",$valmib);
 if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG LACP dot3adAggPortActorAdminKey MIB is ->".print_r($valmib,true)."\n");
 if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok=$objdevice->retcode;
   if ($snmpok) {
   	if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$LACPAdminKey);
        $msg= "";
   }
   else {
        $msg = $objdevice->errormsg;
   }
   if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG LACP AdminKey array ->".print_r($LACPAdminKey,true)."\n");
 }
 if ($snmpok) {
  //  search the other data
  searchin_xmlMIB($a_xmlmib,"dot3adAggPortActorOperKey",$valmib);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG LACP OperKey MIB is ->".print_r($valmib,true)."\n");
  if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok2=$objdevice->retcode;
   if ($snmpok2) {
   	if (sizeof($objdevice->result)>0) convert_index_to_ifindex($objdevice->result,$LACPOperKey);
   }
   else $msg = $objdevice->errormsg;
   if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG LACP OperKey ->".print_r($LACPOperKey,true)."\n");
  }
  searchin_xmlMIB($a_xmlmib,"dot3adAggPortListEntry",$valmib);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG LACP dot3adAggPortListEntry MIB is ->".print_r($valmib,true)."\n");
  if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok3=$objdevice->retcode;
   if ($snmpok3) {
   	if (sizeof($objdevice->result)>0) {
   	  convert_index_to_ifindex($objdevice->result,$LACPPortListEntrytemp);
          $LACPPortListEntry = array();
           foreach ($LACPPortListEntrytemp as $ndx => $str) {
   	    $LACPPortListEntry[$ndx] = array();
   	    $val ="";
   	    $j=0;
   	    for($i = 0; $i < strlen($str); $i++) {
   	       $sbstr =substr($str, $i, 1);
               if (($sbstr!=":")&&($sbstr!=" ")) {
               	 $xval = intval('0x'.$sbstr,16);
               	 if ($xval & 0x8) $val2="1"; else $val2= "0";
               	 if ($val2=="1") 
               	   $LACPPortListEntry[$ndx][$j] =$val2;
               	 $j++;
               	 if ($xval & 0x4) $val2="1"; else $val2= "0";
               	 if ($val2=="1") 
               	   $LACPPortListEntry[$ndx][$j] =$val2;
               	 $j++;
               	 if ($xval & 0x2) $val2="1"; else $val2= "0";
               	 if ($val2=="1") 
                   $LACPPortListEntry[$ndx][$j] =$val2;
               	 $j++;
               	 if ($xval & 0x1) $val2="1"; else $val2= "0";
               	 if ($val2=="1") 
               	   $LACPPortListEntry[$ndx][$j] =$val2;
               	 $j++;
               	}
            }
          }
   	}
   }
   else $msg = $objdevice->errormsg;
   if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG LACP dot3adAggPortListEntry ->".print_r($PortActorAdminState,true)."\n");
  }
  searchin_xmlMIB($a_xmlmib,"dot3adAggPortActorOperState",$valmib);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG LACP PortAttached MIB is ->".print_r($valmib,true)."\n");
  if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib,"i");
   $snmpok3=$objdevice->retcode;
   if ($snmpok3) {
     if (sizeof($objdevice->result)>0) {
     	convert_index_to_ifindex($objdevice->result,$PortActorOperState);
/* dot3adAggPortActorOperState returns
LacpState ::= TEXTUAL-CONVENTION
   STATUS      current
   DESCRIPTION
       "The Actor and Partner State values from the LACPDU."
   SYNTAX      BITS {
                   lacpActivity(0),
                   lacpTimeout(1),
                   aggregation(2),
                   synchronization(3),
                   collecting(4),
                   distributing(5),
                   defaulted(6),
                   expired(7)
               }
*/
      $d=0;
      foreach ($PortActorOperState as $indx => $val) {
      	$dx2="";
        foreach ($LACPPortListEntry as $dx => $val2) {
           $LACPChannelPortList = @$val2;
      	   $isch = (isset($LACPChannelPortList[$d])) ?$LACPChannelPortList[$d]:"0";
      	   if ($isch=="1") $dx2= $dx;
        }
      	$PortActorOperState[$indx] = ((intval($val,10) & 0x20)!=0)?$dx2:"0";
      	$d++;
      }
     }
   }
   else $msg = $objdevice->errormsg;
   if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG LACP PortAttached ->".print_r($PortActorOperState,true)."\n");
  }
  searchin_xmlMIB($a_xmlmib,"dot3adAggPortActorAdminState",$valmib);
  if (($networkclass_isdebug)|| ($networkclass_islog)) networkclass_debug("DEBUG LACP PortActorAdminState MIB is ->".print_r($valmib,true)."\n");
  if (sizeof($valmib)>0) {
   networkclass_give_snmp($objdevice,$valmib);
   $snmpok3=$objdevice->retcode;
   if ($snmpok3) {
     if (sizeof($objdevice->result)>0) {
      convert_index_to_ifindex($objdevice->result,$PortActorAdminState);
      $d=0;
      foreach ($PortActorAdminState as $indx => $val) {
      	$dx2="";
        foreach ($LACPPortListEntry as $dx => $val2) {
           $LACPChannelPortList = @$val2;
      	   $isch = (isset($LACPChannelPortList[$d])) ?$LACPChannelPortList[$d]:"0";
      	   if ($isch=="1") $dx2= $dx;
        }
      	$PortActorAdminState[$indx] = ((intval($val,10) & 0x20)!=0)?$dx2:"0";
      	$d++;
      }
     }
   }
   else $msg = $objdevice->errormsg;
   if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG LACP PortActorAdminState ->".print_r($PortActorAdminState,true)."\n");
  }
 }
 /* convert to ifindex !!!!!*/
 $result["AdminKey"] = @$LACPAdminKey;
 $result["OperKey"] = @$LACPOperKey;
 $result["PortActorAdminState"] = @$PortActorAdminState;
 $result["PortActorOperState"] = @$PortActorOperState;
// $result["LACPPortListEntry"] = @$LACPPortListEntry;
 $objdevice->result= $result;
// if (($networkclass_isdebug)||($networkclass_islog)) networkclass_debug("DEBUG LACP result ->".print_r($result,true)."\n");
 $objdevice->errormsg = $msg;
 $objdevice->retcode = $snmpok;
 return $snmpok;
}

function convert_index_to_ifindex(&$t_array,&$t_array2) {
	/* the last octet of the oid is the index by default */
	$index_parse_regexp = '.*\.([0-9]+)$';
	/* parse the index if required */
	$t_array2=array();
	for ($i=0; $i<sizeof($t_array); $i++) {
		$t_array2[ereg_replace($index_parse_regexp, "\\1", $t_array[$i]["oid"])]=$t_array[$i]["value"];
	}
//	if (sizeof($t_array)>0) {
//		unset($t_array);
//		$t_array=$t_array2;
//	}
}

function remove_oid_for_find_index($oid,&$t_array,&$t_array2) {
	/* the part over the oid part is the index by default */
	$oid_parsed= preg_replace('/^\./', '', $oid); // remove the first .
	for ($i=0; ($i<sizeof($t_array)); $i++) {
	  $indx =$t_array[$i]["oid"];
	  if (preg_match('/^iso\./',$indx)>0) $oid_parsed=preg_replace('/^1/', 'iso', $oid_parsed);
          break;
	}
	$oid_parsed= preg_replace('/\./', '\.', $oid_parsed);
	$index_parse_regexp = '/'.$oid_parsed.'\.([0-9,\.]+)$/';
	/* parse the index if required */
	$t_array2=array();
	for ($i=0; $i<sizeof($t_array); $i++) {
		$newindex = preg_replace($index_parse_regexp, "\\1", $t_array[$i]["oid"]);
		$t_array2[$newindex]=$t_array[$i]["value"];
	}
}

// Load the xml MIB given in $a_xmlmmib and search the value of $mib, returned in $valmib
// $a_xmlmmib = array(array("source",filexml1), array(), ....)
function searchin_xmlMIB(&$a_xmlmib,$mib,&$valmib) {
  global $sharednetworkclasspath;
   $rpathMIBXML= "MIBXML/"; if (isset($sharednetworkclasspath)) $rpathMIBXML=$sharednetworkclasspath."/".$rpathMIBXML;
   networkclass_debug("DEBUG searchin_xmlMIB: a_xmlmib ->".print_r($a_xmlmib,true)."\n");
   $valmib=array();
   if (sizeof($a_xmlmib)>0) {
     foreach($a_xmlmib as $tabxml) {
   	unset($arrayOfMIB);
   	$filexml="";
   	if (array_key_exists("source",$tabxml)) $filexml=$tabxml["source"];
//   networkclass_debug("DEBUG searchin_xmlMIB: filexml ->".$filexml." for mib=$mib\n");
   	if ($filexml!="")
            $arrayOfMIB= networkclass_get_xml_array($rpathMIBXML.$filexml);
         else $arrayOfMIB=array();
//   networkclass_debug("DEBUG searchin_xmlMIB: arrayOfMIB ->".print_r($arrayOfMIB,true)."\n");
        if ((sizeof($arrayOfMIB)>0) && (array_key_exists("fields",$arrayOfMIB))) {
   	  $mibfields=$arrayOfMIB["fields"];
   	  if ((sizeof($mibfields)>0)&& array_key_exists($mib,$mibfields)) {
   	     $valmib= $mibfields[$mib];
   	     break;
	  }
        }
     }
   }
}

// Load the xml MIB given in $a_xmlmmib and search the value of $mib, returned in $valmib
// $a_xmlmmib = array(array("source",filexml1), array(), ....)
function searchByValuein_xmlMIB(&$a_xmlmib,$value,&$valmib) {
  global $sharednetworkclasspath;
   $rpathMIBXML= "MIBXML/"; if (isset($sharednetworkclasspath)) $rpathMIBXML=$sharednetworkclasspath."/".$rpathMIBXML;
   networkclass_debug("DEBUG searchin_xmlMIB: a_xmlmib ->".print_r($a_xmlmib,true)."\n");
   $valmib=array();
   if (sizeof($a_xmlmib)>0) {
     foreach($a_xmlmib as $tabxml) {
   	unset($arrayOfMIB);
   	$filexml="";
   	if (array_key_exists("source",$tabxml)) $filexml=$tabxml["source"];
//   networkclass_debug("DEBUG searchin_xmlMIB: filexml ->".$filexml." for mib=$mib\n");
   	if ($filexml!="")
            $arrayOfMIB= networkclass_get_xml_array($rpathMIBXML.$filexml);
         else $arrayOfMIB=array();
//   networkclass_debug("DEBUG searchin_xmlMIB: arrayOfMIB ->".print_r($arrayOfMIB,true)."\n");
        if ((sizeof($arrayOfMIB)>0) && (array_key_exists("fields",$arrayOfMIB))) {
   	  $mibfields=$arrayOfMIB["fields"];
   	  if (sizeof($mibfields)>0) {
   	     $result = array_search ( $value, $mibfields);
   	     if ($result!=false){
   	     	 $valmib= $mibfields[$result];
   	         break;
   	     }
	  }
        }
     }
   }
}

function networkclass_searchinMIB($mib,&$valmib) {
  global $networkclass_max_class_load,$networkclass_tab_class_loaded;
  global $sharednetworkclasspath;
   $rpathMIBXML= "MIBXML/"; if (isset($sharednetworkclasspath)) $rpathMIBXML=$sharednetworkclasspath."/".$rpathMIBXML;
   $fileMIBtoLoadxml="MIBtoLoad.xml";
   $arrayOfMIBtoLoad= networkclass_get_xml_array($rpathMIBXML.$fileMIBtoLoadxml);
//    networkclass_debug("DEBUG arrayOfMIBtoLoad ->".print_r($arrayOfMIBtoLoad,true)."\n");
   $valmib=array();
   if (array_key_exists("fields",$arrayOfMIBtoLoad)) {
     foreach($arrayOfMIBtoLoad["fields"] as $tabxml) {
   	unset($arrayOfMIB);
   	$filexml="";
   	if (array_key_exists("source",$tabxml)) $filexml=$tabxml["source"];
   	if ($filexml!="") {
   	  if (array_key_exists($filexml,$networkclass_tab_class_loaded))
             $arrayOfMIB= &$networkclass_tab_class_loaded[$filexml];
   	  else {
            $arrayOfMIB= networkclass_get_xml_array($rpathMIBXML.$filexml);
            if (sizeof($networkclass_max_class_load)<$networkclass_max_class_load) 
               $networkclass_tab_class_loaded[$filexml]=$arrayOfMIB;
          }
        } else $arrayOfMIB=array();
        if ((sizeof($arrayOfMIB)>0) && (array_key_exists("fields",$arrayOfMIB))) {
   	  $mibfields=$arrayOfMIB["fields"];
   	  if ((sizeof($mibfields)>0)&& array_key_exists($mib,$mibfields)) {
   	     $valmib= $mibfields[$mib];
   	     break;
	  }
        }
     }
   }
}

function networkclass_read_mib($filexml) {
  global $sharednetworkclasspath;
//to force a manual debug
//$objdevice->setdebug(false,true);
//networkclass_debug("DEBUG networkclass_read_mib\n");
  if ($handle = opendir($sharednetworkclasspath)) 
  {
   networkclass_debug("DEBUG networkclass_read_mib sharednetworkclasspath=".$sharednetworkclasspath."\n");
           networkclass_debug("DEBUG networkclass_read_mib sharednetworkclasspath=".$sharednetworkclasspath."\n");
   if(snmp_read_mib($sharednetworkclasspath.'/'.$filexml)) $ok=1;
   else $ok=0;
  }
  closedir($handle);
}

function networkclass_give_snmp(&$objdevice,$valmib,$type="s") {
//$objdevice->setdebug(false,true);
   $snmpok = 0;
   $error= "";
   if (array_key_exists("oid",$valmib)) $oid= $valmib["oid"];
   else $oid= "";
   if (array_key_exists("method",$valmib)) $method= $valmib["method"];
   else $method="";
   if (array_key_exists("name",$valmib)) $mib= $valmib["name"];
   else $mib=$oid;
   $tanslate="";
   if (array_key_exists("source",$valmib)) {
   	$tanslate= $valmib["source"];
   	if (strpos($tanslate,"OID/REGEXP:" )!==false) $tanslate=str_replace("OID/REGEXP:", "", $tanslate);
   	else $tanslate="";
   }
   if ($oid!="") {
    if ($method=="get") {
     $ret_val = $objdevice->snmp->get($oid.".0");
     $result  = $objdevice->snmp->result;
     $snmpok  = $objdevice->snmp->retcode;
     $error   = $objdevice->snmp->errormsg;
     if ($snmpok && ($tanslate!="")) ereg_replace($tanslate, "\\1", $result);
    } else if ($method=="walk") {
     $objdevice->snmp->walk($oid,$type);
     $result = $objdevice->snmp->result;
     $error= $objdevice->snmp->errormsg;
     $snmpok = $objdevice->snmp->retcode;
     if ($snmpok && ($tanslate!="")) {
       for ($i=0; $i<sizeof($result); $i++) {
          $result[$i]["value"] = ereg_replace($tanslate, "\\1", $result[$i]["oid"]);
       }
     }
    } elseif ($method=="getnext") {
     $objdevice->snmp->getnext($oid);
     $result = $objdevice->snmp->result;
     $error= $objdevice->snmp->errormsg;
     $snmpok =$objdevice->snmp->retcode;
     if ($snmpok && ($tanslate!="")) ereg_replace($tanslate, "\\1", $result);
    }
   } else {
       $result  = "";
       $error= "Error this function '".$mib."' do not exist or not implemented.";
       $snmpok = 0;
   }
   $objdevice->result  = $result;
   $objdevice->retcode = $snmpok;
   if (!$snmpok)  $objdevice->errormsg= "Error $method ".$error;
   else $objdevice->errormsg= $error;
   return $snmpok;
}

function networkclass_discovervendor(&$objdevice,&$snmp_value)
{
        networkclass_searchinMIB("sysObjectID",$valmib);
	$snmpok = networkclass_give_snmp($objdevice,$valmib);
        $snmp_value = $objdevice->snmp->result;
//        $snmpok = $objdevice->snmp->retcode;
        $error= $objdevice->snmp->errormsg;
	if ($snmpok) {
		$snmp_value = trim(strtr($snmp_value,"\""," "));
		$search = '/oid\s*:\s*/i';
		$replace = '';
		$snmp_value= preg_replace($search, $replace, $snmp_value);
		$search = '/\.(iso)(\..*)*/';
		$replace = '.1$2';
		$snmp_value= preg_replace($search, $replace, $snmp_value);
		$search = '/\.(\d+)\.(\d+)\.(\d+)\.(\d+)\.(\d+)\.(\d+)\.(\d+)(\..*)*/';
		$replace = '.$1.$2.$3.$4.$5.$6.$7';
		$snmp_value= preg_replace($search, $replace, $snmp_value);
		$ok = true;
	}
	$objdevice->result  = &$snmp_value;
	$objdevice->retcode = $snmpok;
	$objdevice->errormsg= $error;
	return $snmpok;
}

function networkclass_get_xml_array($filexml) {
	if (!file_exists($filexml)) {
		$message = "Could not find data query XML file at '$filexml'\n";
		networkclass_debug($message);
		return false;
	}
	else $message = "Found data query XML file at '$filexml'\n";
	networkclass_debug($message);
	$data = implode("",file($filexml));
//   networkclass_debug("DEBUG networkclass_get_xml_array: filexml ->".$data." \n");
	return xml2array($data);
}


?>